/* ------------------------------------------------------------------ 

	Title: BRWindow

	Description:  Shows resource information

	Author :
		Stephen Wardlaw, M.D.
		Medical R&D
		Highrock
		Lyme, CT 06371

	Edit History:	
		22 November, 1999
			Modified to add BPicture saving
		05 September, 1999
			Created
------------------------------------------------------------------ */ 
#include "BRWindow.h"

#include "U_Conversions.h"
#include <StorageKit.h>
#include <BitmapStream.h>
#include <TranslatorRoster.h>
#include <stdio.h>
#include <string.h>

const uint32 open_rsrc	= 'rOpn';
const uint32 new_rsrc	= 'rNew';
const uint32 close_rsrc	= 'rCls';
const uint32 res_type	= 'rTyp';

const char bitmap_str[16] = "Save As BBitmap";
const char picture_str[32] = "Save As BPicture";
const char add_bmp_str[16] = "Add BBitmap"B_UTF8_ELLIPSIS;
const char add_pic_str[16] = "Add BPicture"B_UTF8_ELLIPSIS;

const char resource_type[32] = "application/x-be-resource";
// -------------------------------------------------------------------
//	* CONSTRUCTOR
// -------------------------------------------------------------------
BRWindow::BRWindow(BRect frame, const char* name = "Bitmap2Rsrc"): WTextWindow(frame,name)
{
	BMenuItem* quitItem = mFileMenu->RemoveItem((int32)0);
	mNewItem = new BMenuItem("New Resource",new BMessage(new_rsrc));
	mFileMenu->AddItem(mNewItem);
	mOpenItem = new BMenuItem("Open Resource",new BMessage(open_rsrc));
	mFileMenu->AddItem(mOpenItem);
	mCloseItem = new BMenuItem("Close Resource",new BMessage(close_rsrc));
	mFileMenu->AddItem(mCloseItem);
	mFileMenu->AddSeparatorItem();
	mAddItem = new BMenuItem(add_bmp_str,new BMessage(open_rsrc));
	mFileMenu->AddItem(mAddItem);
	mFileMenu->AddSeparatorItem();
	mFileMenu->AddItem(quitItem);	// Replace 'Quit'
	BMenu* dataMenu = new BMenu("Data");
	mResType = new BMenuItem(picture_str, new BMessage(res_type));
	dataMenu->AddItem(mResType);
	mMenuBar->AddItem(dataMenu);
	
	// Construct the open file panel
	mResources = NULL;
	mResIndx = 0;
	mOpenPanel = new BFilePanel(B_OPEN_PANEL);
	BMessenger* theMessenger = new BMessenger(this);
	mOpenPanel->SetTarget(*theMessenger);
	mSavePanel = new BFilePanel(B_SAVE_PANEL);
	mSavePanel->SetSaveText("Untitled.rsrc");
	mSavePanel->SetTarget(*theMessenger);
	delete theMessenger;	
	
	// Add a view for picture generation
	mPictView = new BView(BRect(1000,1000,2000,2000),"",B_FOLLOW_NONE,0);
	AddChild(mPictView);
	mArchivedObj = new BMessage(B_ARCHIVED_OBJECT);
	mSaveAsBitmap = true;
}

BRWindow::~BRWindow()
{
	delete mResources;
	delete mOpenPanel;
	delete mArchivedObj;
}
// -------------------------------------------------------------------
//	* Public Methods
// -------------------------------------------------------------------
void
BRWindow::MessageReceived(BMessage* theMesg)
{
	int32 resID;
	const char* resName;
	
	switch (theMesg->what) {
	
		case B_REFS_RECEIVED :
			if(mResources == NULL) {
				ClearText();
				OpenFile(theMesg);
			} else {
				GetBitmap(theMesg);
			}
			break;
			
		case B_SAVE_REQUESTED :	// Creating a new resource
			mResIndx = 0;
			SaveAsFile(theMesg);
			break;

			
		case add_bmp :
			resID = theMesg->FindInt32(res_id_str);
			resName = theMesg->FindString(res_name_str);
			AddBitmap(resID,resName);
			break;
			
		case new_rsrc :
			mSavePanel->Show();
			break;
			
		case open_rsrc :
			mOpenPanel->Show();
			break;
			
		case close_rsrc :
			delete mResources;
			mResources = NULL;
			ClearText();
			break;
			
		case res_type:
			mSaveAsBitmap = !mSaveAsBitmap;
			if(mSaveAsBitmap) {
				mResType->SetLabel(picture_str);
				mAddItem->SetLabel(add_bmp_str);
			} else {
				mResType->SetLabel(bitmap_str);
				mAddItem->SetLabel(add_pic_str);
			}
			break;
			
		default :
			WTextWindow::MessageReceived(theMesg);
	}
}		
// -------------------------------------------------------------------
void
BRWindow::RefsReceived(BMessage* theMesg)
{
}
// -------------------------------------------------------------------
//	* Protected Methods
// -------------------------------------------------------------------
void
BRWindow::AddBitmap(int32 resID, const char* rName)
{	
	if(!mSaveAsBitmap) {	// Convert to a BPicture
		Lock();
		BBitmap* theBitmap = new BBitmap(mArchivedObj);
		mPictView->BeginPicture(new BPicture());
		mPictView->DrawBitmap(theBitmap);
		BPicture* thePict = mPictView->EndPicture();
		delete theBitmap;
		mArchivedObj->MakeEmpty();
		thePict->Archive(mArchivedObj);
		delete thePict;
		Unlock();
	}
	
	size_t size = mArchivedObj->FlattenedSize();
	char* data = new char[size];
	mArchivedObj->Flatten(data,size);
	mResources->AddResource(B_RAW_TYPE,resID,data,size,rName);
	delete data;
		
	if(mSaveAsBitmap) {
		SingleLine("New BBitmap Added:");
	} else {
		SingleLine("New BPicture Added:");
	}
	mArchivedObj->MakeEmpty();
	ShowResource(mResIndx++);
}
// -------------------------------------------------------------------
void
BRWindow::GetBitmap(BMessage* theMesg)
{
	entry_ref theRef;
	mArchivedObj->MakeEmpty();	// Delete any old contents
	if(theMesg->FindRef("refs",&theRef) == B_OK) {
		BFile theFile(&theRef,B_READ_ONLY);
		if(theFile.InitCheck() == B_OK) {
			BTranslatorRoster *roster = BTranslatorRoster::Default(); 
			BBitmapStream stream; 
			BBitmap* theBitmap = NULL;
			if (roster->Translate(&theFile, NULL, NULL, &stream,B_TRANSLATOR_BITMAP) == B_OK) { 
				stream.DetachBitmap(&theBitmap); // BBitmap from file
				theBitmap->Archive(mArchivedObj); // Archive BBitmap into BMessage
				new BREditWindow(theRef.name);	// Put up new window
			} else {
				BAlert* theAlert = new BAlert("BmAlert","Can't Translate Bitmap.","OK");
				theAlert->Go();
			}
		} else {	// Of if(theFile.InitCheck
			BAlert* theAlert = new BAlert("BmAlert","Can't Open Bitmap File.","OK");
			theAlert->Go();
		}
	}	// Of if(theMesg->FindRef
}
// -------------------------------------------------------------------
void
BRWindow::ListResources(const char* fileName)
{
	int32 mResIndx = 0;
	Append("Resources Found In: ");
	SingleLine(fileName);
	while (ShowResource(mResIndx++)) ; 
}
// -------------------------------------------------------------------
void
BRWindow::MenusBeginning()
{
	bool haveFile = mResources != NULL;
	mAddItem->SetEnabled(haveFile);
	mOpenItem->SetEnabled(!haveFile);
	mNewItem->SetEnabled(!haveFile);
	mCloseItem->SetEnabled(haveFile);
}
// -------------------------------------------------------------------
// Open a new file 
void
BRWindow::OpenFile(BMessage* theMesg)
{
	// Start a new file under a different name
	mResources = new BResources();
	entry_ref theRef;
	bool completed = false;
	if(theMesg->FindRef("refs",&theRef) == B_OK) {
		BFile theFile(&theRef,B_READ_WRITE);
		if(theFile.InitCheck() == B_OK) {
			if(mResources->SetTo(&theFile,false) == B_OK) {
				completed = true;
				ListResources(theRef.name);
			}	// Of if SetTo()
		}	// Of if(theFile.InitCheck
	}	// Of if(theMesg->FindRef
	if(!completed) {
		delete mResources;
		mResources = NULL;
		char errStr[128];
		::strcpy(errStr,"The File Named ");
		::strcat(errStr,theRef.name);
		::strcat(errStr," Could Not Be Opened");
		BAlert* theAlert = new BAlert("FileError",errStr,"OK");
		theAlert->Go();
	}
}		
// -------------------------------------------------------------------
void
BRWindow::SaveAsFile(BMessage* theMesg)
{
	// Start a new file under a different name
	entry_ref theRef;
	BDirectory theDir;
	BPath thePath;
	const char* fString;
	bool completed = false;
	if(theMesg->FindRef("directory",&theRef) == B_OK) {
		fString = theMesg->FindString("name");
		if(fString[0] > 0) {
			theDir.SetTo(&theRef);
			if(theDir.InitCheck() == B_OK) {
				thePath.SetTo(&theDir,fString);
				if(thePath.InitCheck() == B_OK) {
					BFile theFile(thePath.Path(),B_READ_WRITE|B_CREATE_FILE);
					if(theFile.InitCheck() == B_OK) {
						mResources = new BResources();
						if(mResources->SetTo(&theFile,true) == B_OK) {
							completed = true;
							// Set the MIME type as a "resource" file
							BNodeInfo theInfo;
							theInfo.SetTo(&theFile);
							theInfo.SetType(resource_type);
							Append("Created File: ");
							SingleLine(fString);
						}	// Of if SetTo()
					}	// Of if(theFile.InitCheck()
				}	// Of if(thePath.InitCheck()
			}
		}
	}
	if(!completed) {
		delete mResources;
		mResources = NULL;
		char errStr[128];
		::strcpy(errStr,"The File Named ");
		::strcat(errStr,fString);
		::strcat(errStr," Could Not Be Created.");
		BAlert* theAlert = new BAlert("FileError",errStr,"OK");
		theAlert->Go();
	}
}
// ------------------------------------------------------------------
bool
BRWindow::ShowResource(int32 index)
{
	char outStr[255];
	size_t length;
	const char*	rName;
	type_code	resType;
	int32	resID;
	bool found = mResources->GetResourceInfo(index,&resType,&resID,&rName,&length);
	if(found) {
		sprintf(outStr,"Index = %3ld  Length = %ld",mResIndx,length);
		SingleLine(outStr);
		Append("Type  = ");
		::TypeString(resType,outStr);
		SingleLine(outStr);
		Append("ID    = ");
		::Long2Str(resID,outStr);
		SingleLine(outStr);
		sprintf(outStr,"Name  = %s",rName);
		SingleLine(outStr);
		NewLine();
	}
	return found;
}		
// ------------------------------------------------------------------
